@@ -21,11 +21,11 @@ struct starter_conn;
 struct starter_config;
 
 int starter_whack_add_conn (struct starter_config *cfg, struct starter_conn *conn);
-int starter_whack_del_conn (struct starter_conn *conn);
-int starter_whack_route_conn (struct starter_conn *conn);
-int starter_whack_initiate_conn (struct starter_conn *conn);
-int starter_whack_listen (void);
-int starter_whack_shutdown (void);
+int starter_whack_del_conn (struct starter_config *cfg, struct starter_conn *conn);
+int starter_whack_route_conn (struct starter_config *cfg, struct starter_conn *conn);
+int starter_whack_initiate_conn (struct starter_config *cfg, struct starter_conn *conn);
+int starter_whack_listen (struct starter_config *cfg);
+int starter_whack_shutdown (struct starter_config *cfg);
 
 #endif /* _STARTER_WHACK_H_ */
 
