@@ -16,7 +16,6 @@
  */
 
 #include <stdlib.h>
-#include <stdarg.h>
 #include <stdio.h>
 #include <string.h>
 #include <limits.h>
@@ -63,10 +62,7 @@ static void default_values (struct starter_config *cfg)
 	cfg->setup.options[KBF_UNIQUEIDS]= FALSE;
 	cfg->conn_default.options[KBF_TYPE] = KS_TUNNEL;
 
-	cfg->conn_default.policy =
-	    POLICY_RSASIG|POLICY_TUNNEL|POLICY_ENCRYPT|POLICY_PFS;
-	
-	cfg->conn_default.policy |= POLICY_IKEV2_ALLOW;
+	cfg->conn_default.policy = POLICY_RSASIG|POLICY_TUNNEL|POLICY_ENCRYPT|POLICY_PFS;
 
 	cfg->conn_default.options[KBF_IKELIFETIME] = OAKLEY_ISAKMP_SA_LIFETIME_DEFAULT;
 	cfg->conn_default.options[KBF_SALIFETIME]  = SA_LIFE_DURATION_DEFAULT;
@@ -79,12 +75,9 @@ static void default_values (struct starter_config *cfg)
 
 	cfg->conn_default.left.addr_family = AF_INET;
 	anyaddr(AF_INET, &cfg->conn_default.left.addr);
-	cfg->conn_default.left.nexttype  = KH_NOTSET;
 	anyaddr(AF_INET, &cfg->conn_default.left.nexthop);
-
 	cfg->conn_default.right.addr_family = AF_INET;
 	anyaddr(AF_INET, &cfg->conn_default.right.addr);
-	cfg->conn_default.right.nexttype = KH_NOTSET;
 	anyaddr(AF_INET, &cfg->conn_default.right.nexthop);
 
 	/* default is to look in DNS */
@@ -98,35 +91,11 @@ static void default_values (struct starter_config *cfg)
 	cfg->ctlbase = clone_str(CTL_FILE, "default base");
 }
 
-/* format error, and append to string of errors */
-int error_append(char **perr, const char *fmt, ...) 
-{
-    va_list args;
-
-    if(perr) {
-	char *nerr; 
-	int len; 
-	
-	va_start(args, fmt);
-	vsnprintf(_tmp_err, sizeof(_tmp_err)-1, fmt, args);
-	va_end(args);
-	
-	len = 1+ strlen(_tmp_err) + (*perr ? strlen(*perr) : 0);	
-	nerr = xmalloc(len);
-	nerr[0]='\0';
-	if(*perr) strcpy(nerr, *perr); 
-	strcat(nerr, _tmp_err);			
-	
-	if(*perr) free(*perr);
-	*perr = nerr;
-
-	return 1;
-    }
-    return 0;
-}
-
-#define ERR_FOUND(args...) do { err += error_append(&err_str, ##args); } while(0)
-
+#define ERR_FOUND(args...) \
+	{ if (perr && (*perr==NULL)) { \
+		snprintf(_tmp_err, sizeof(_tmp_err)-1, ## args); \
+		*perr = xstrdup(_tmp_err); } \
+	err++; }
 
 #define KW_POLICY_FLAG(val,fl) if(conn->options_set[val]) \
         { if(conn->options[val]) \
@@ -235,7 +204,6 @@ static int load_setup (struct starter_config *cfg
 		break;
 
 	    case kt_appendstring:
-	    case kt_appendlist:
 		assert(kw->keyword.keydef->field < KEY_STRINGS_MAX);
 		if(!cfg->setup.strings[kw->keyword.keydef->field])
 		{
@@ -279,9 +247,6 @@ static int load_setup (struct starter_config *cfg
 	    case kt_idtype:
 		err++;
 		break;
-
-	    case kt_comment:
-		break;
 	    }
 	}
 		
@@ -309,12 +274,9 @@ static int load_setup (struct starter_config *cfg
  */
 static int validate_end(struct starter_conn *conn_st
 			, struct starter_end *end
-			, bool left
-			, bool resolvip UNUSED
-			, err_t *perr)
+			, bool left, err_t *perr)
 {
     err_t er = NULL;
-    char *err_str = NULL;
     const char *leftright=(left ? "left" : "right");
     int err=0;
 
@@ -324,6 +286,15 @@ static int validate_end(struct starter_conn *conn_st
 
     end->addrtype=end->options[KNCF_IP];
 
+    /* validate the KSCF_ID */
+    if(end->strings[KSCF_ID] != NULL)
+    {
+	char *value = end->strings[KSCF_ID];
+	
+	if (end->id) free(end->id);
+	end->id = xstrdup(value);
+    } 
+
     /* validate the KSCF_IP/KNCF_IP */
     switch(end->addrtype)
     {
@@ -345,12 +316,8 @@ static int validate_end(struct starter_conn *conn_st
     case KH_IPADDR:
 	assert(end->strings[KSCF_IP] != NULL);
 
-	er = ttoaddr_num(end->strings[KNCF_IP], 0, AF_INET, &(end->addr));
-	if(er) {
-	    /* not numeric, so set the type to the string type */
-	    end->addrtype = KH_IPHOSTNAME; 
-	}
-
+	er = ttoaddr(end->strings[KNCF_IP], 0, AF_INET, &(end->addr));
+	if (er) ERR_FOUND("bad addr %s=%s [%s]", leftright, end->strings[KNCF_IP], er);
         if(end->id == NULL) {
             char idbuf[ADDRTOT_BUF];
             addrtot(&end->addr, 0, idbuf, sizeof(idbuf));
@@ -371,10 +338,6 @@ static int validate_end(struct starter_conn *conn_st
 	conn_st->policy |= POLICY_GROUP;
 	break;
 	
-    case KH_IPHOSTNAME:
-	/* generally, this doesn't show up at this stage */
-	break;
-
     case KH_DEFAULTROUTE:
 	break;
 
@@ -408,21 +371,13 @@ static int validate_end(struct starter_conn *conn_st
     if(end->strings_set[KSCF_NEXTHOP])
     {
 	char *value = end->strings[KSCF_NEXTHOP];
-
-	if(strcasecmp(value, "%defaultroute")==0) {
-	    end->nexttype=KH_DEFAULTROUTE;
-	} else {
-	    er = ttoaddr(value, 0, AF_INET, &(end->nexthop));
-	    if (er) ERR_FOUND("bad addr %snexthop=%s [%s]", leftright, value, er);
-
-	    end->nexttype = KH_IPADDR;
-	}
+	
+	er = ttoaddr(value, 0, AF_INET, &(end->nexthop));
+	if (er) ERR_FOUND("bad addr %snexthop=%s [%s]", leftright, value, er);
     } else {
-#if 0
 	if(conn_st->policy & POLICY_OPPO) {
 	    end->nexttype = KH_DEFAULTROUTE;
 	}
-#endif
 	anyaddr(AF_INET, &end->nexthop);
     }
 
@@ -497,6 +452,10 @@ static int validate_end(struct starter_conn *conn_st
 	end->updown = xstrdup(end->strings[KSCF_UPDOWN]);
     }
 
+    if(end->strings_set[KSCF_UPDOWN]) {
+	end->updown = xstrdup(end->strings[KSCF_UPDOWN]);
+    }
+
     if(end->strings_set[KSCF_PROTOPORT]) {
 	err_t ugh;
 	char *value = end->strings[KSCF_PROTOPORT];
@@ -514,7 +473,6 @@ static int validate_end(struct starter_conn *conn_st
     KSCF_MAX          = 19
 */
 
-    if(err) *perr = err_str;
     return err;
 }
 
@@ -525,15 +483,12 @@ static int validate_end(struct starter_conn *conn_st
  *
  * @param conn a connection definition
  * @param sl a section_list
- * @param assigned_value is set to either k_set, or k_default.
- *        k_default is used when we are loading a conn that should be
- *        considered to be a "default" value, and that replacing this
- *        value is considered acceptable.
+ * @param permitreplace boolean (can we replace this conn?)
  * @return bool 0 if successfull
  */
 bool translate_conn (struct starter_conn *conn
 		     , struct section_list *sl
-		     , enum keyword_set   assigned_value
+		     , bool permitreplace
 		     , err_t *error)
 {
     unsigned int err, field;
@@ -603,23 +558,26 @@ bool translate_conn (struct starter_conn *conn
 	case kt_idtype:
 	    /* all treated as strings for now */
 	    assert(kw->keyword.keydef->field < KEY_STRINGS_MAX);
-	    if((*set_strings)[field] == k_set)
+	    if((*set_strings)[field])
 	    {
-		*error = _tmp_err;
-		
-		snprintf(_tmp_err, sizeof(_tmp_err)
-			 , "duplicate key '%s' in conn %s while processing def %s"
-			 , kw->keyword.keydef->keyname
-			 , conn->name
-			 , sl->name);
-		
-		starter_log(LOG_LEVEL_INFO, _tmp_err);
-		if(kw->keyword.string == NULL
-		   || (*the_strings)[field] == NULL
-		   || strcmp(kw->keyword.string, (*the_strings)[field])!=0)
+		if(!permitreplace)
 		{
-		    err++;
-		    break;
+		    *error = _tmp_err;
+
+		    snprintf(_tmp_err, sizeof(_tmp_err)
+			     , "duplicate key '%s' in conn %s while processing def %s"
+			     , kw->keyword.keydef->keyname
+			     , conn->name
+			     , sl->name);
+		    
+		    starter_log(LOG_LEVEL_INFO, _tmp_err);
+		    if(kw->keyword.string == NULL
+		       || (*the_strings)[field] == NULL
+		       || strcmp(kw->keyword.string, (*the_strings)[field])!=0)
+		    {
+			err++;
+			break;
+		    }
 		}
 	    }
 	    if((*the_strings)[field])
@@ -628,11 +586,10 @@ bool translate_conn (struct starter_conn *conn
 	    }
 	    
 	    (*the_strings)[field] = xstrdup(kw->string);
-	    (*set_strings)[field] = assigned_value;
+	    (*set_strings)[field] = TRUE;
 	    break;
 	    
 	case kt_appendstring:
-	case kt_appendlist:
 	    /* implicitely, this field can have multiple values */
 	    assert(kw->keyword.keydef->field < KEY_STRINGS_MAX);
 	    if(!(*the_strings)[field])
@@ -661,27 +618,31 @@ bool translate_conn (struct starter_conn *conn
 	    assert(field < KEY_STRINGS_MAX);
 	    assert(field < KEY_NUMERIC_MAX);
 	    
-	    if((*set_options)[field] == k_set)
+	    if((*set_options)[field])
 	    {
-		*error = _tmp_err;
-		snprintf(_tmp_err, sizeof(_tmp_err)
-			 , "duplicate key '%s' in conn %s while processing def %s"
-			 , kw->keyword.keydef->keyname
-			 , conn->name
-			 , sl->name);
-		
-		starter_log(LOG_LEVEL_INFO, _tmp_err);
-		
-		/* only fatal if we try to change values */
-		if((*the_options)[field] != kw->number
-		   || !((*the_options)[field] == LOOSE_ENUM_OTHER
-			&& kw->number == LOOSE_ENUM_OTHER
-			&& kw->keyword.string != NULL
-			&& (*the_strings)[field] != NULL
-			&& strcmp(kw->keyword.string, (*the_strings)[field])==0))
+		if(!permitreplace)
 		{
-		    err++;
-		    break;
+		    *error = _tmp_err;
+
+		    snprintf(_tmp_err, sizeof(_tmp_err)
+				, "duplicate key '%s' in conn %s while processing def %s"
+				, kw->keyword.keydef->keyname
+				, conn->name
+				, sl->name);
+
+		    starter_log(LOG_LEVEL_INFO, _tmp_err);
+
+		    /* only fatal if we try to change values */
+		    if((*the_options)[field] != kw->number
+		       || !((*the_options)[field] == LOOSE_ENUM_OTHER
+			    && kw->number == LOOSE_ENUM_OTHER
+			    && kw->keyword.string != NULL
+			    && (*the_strings)[field] != NULL
+			    && strcmp(kw->keyword.string, (*the_strings)[field])==0))
+		    {
+			err++;
+			break;
+		    }
 		}
 	    }
 
@@ -692,7 +653,7 @@ bool translate_conn (struct starter_conn *conn
 		if((*the_strings)[field]) free((*the_strings)[field]);
 		(*the_strings)[field] = xstrdup(kw->keyword.string);
 	    } 
-	    (*set_options)[field] = assigned_value;
+	    (*set_options)[field] = TRUE;
 	    break;
 	    
 	case kt_list:
@@ -705,19 +666,23 @@ bool translate_conn (struct starter_conn *conn
 	    /* all treated as a number for now */
 	    assert(field < KEY_NUMERIC_MAX);
 
-	    if((*set_options)[field] == k_set)
+	    if((*set_options)[field])
 	    {
-		*error = _tmp_err;
-		snprintf(_tmp_err, sizeof(_tmp_err)
-			 , "duplicate key '%s' in conn %s while processing def %s"
-			 , kw->keyword.keydef->keyname
-			 , conn->name
-			 , sl->name);
-		starter_log(LOG_LEVEL_INFO, _tmp_err);
-		if((*the_options)[field] != kw->number)
+		if(!permitreplace)
 		{
-		    err++;
-		    break;
+		    *error = _tmp_err;
+
+		    snprintf(_tmp_err, sizeof(_tmp_err)
+			     , "duplicate key '%s' in conn %s while processing def %s"
+			     , kw->keyword.keydef->keyname
+			     , conn->name
+			     , sl->name);
+		    starter_log(LOG_LEVEL_INFO, _tmp_err);
+		    if((*the_options)[field] != kw->number)
+		    {
+			err++;
+			break;
+		    }
 		}
 	    }
 
@@ -726,10 +691,7 @@ bool translate_conn (struct starter_conn *conn
 			kw->keyword.keydef->keyname, field, kw->number);
 #endif
 	    (*the_options)[field] = kw->number;
-	    (*set_options)[field] = assigned_value;
-	    break;
-	    
-	case kt_comment:
+	    (*set_options)[field] = TRUE;
 	    break;
 	}
     }
@@ -737,29 +699,21 @@ bool translate_conn (struct starter_conn *conn
 }
 
 
-void move_comment_list(struct starter_comments_list *to,
-		       struct starter_comments_list *from)
-{
-    struct starter_comments *sc, *scnext;
-    
-    for(sc = from->tqh_first;
-	sc != NULL;
-	sc = scnext) {
-	scnext = sc->link.tqe_next;
-	TAILQ_REMOVE(from, sc,link);
-	TAILQ_INSERT_TAIL(to, sc,link);
-    }
-}
-
 static int load_conn_basic(struct starter_conn *conn
 			   , struct section_list *sl
-			   , enum keyword_set assigned_value
 			   , err_t *perr)
 {
     int err;
 
-    /*turn all of the keyword/value pairs into options/strings in left/right */
-    err = translate_conn(conn, sl, assigned_value, perr);
+    memset(conn->strings_set, 0, sizeof(conn->strings_set));
+    memset(conn->options_set, 0, sizeof(conn->options_set));
+    memset(conn->left.strings_set, 0, sizeof(conn->left.strings_set));
+    memset(conn->left.options_set, 0, sizeof(conn->left.options_set));
+    memset(conn->right.strings_set, 0, sizeof(conn->left.strings_set));
+    memset(conn->right.options_set, 0, sizeof(conn->left.options_set));
+
+    /* turn all of the keyword/value pairs into options/strings in left/right */
+    err = translate_conn(conn, sl, TRUE, perr);
 
     return err;
 }
@@ -771,8 +725,6 @@ static int load_conn (struct starter_config *cfg
 		      , struct config_parsed *cfgp
 		      , struct section_list *sl
 		      , bool alsoprocessing
-		      , bool defaultconn
-		      , bool resolvip
 		      , err_t *perr)
 {
     unsigned int err;
@@ -785,10 +737,7 @@ static int load_conn (struct starter_config *cfg
 	
     err = 0;
 
-    err += load_conn_basic(conn, sl, defaultconn ? k_default : k_set, perr);
-
-    move_comment_list(&conn->comments, &sl->comments);
-
+    err += load_conn_basic(conn, sl, perr);
     if(err) return err;
 
     if(conn->strings[KSF_ALSO] != NULL
@@ -837,8 +786,8 @@ static int load_conn (struct starter_config *cfg
 			, conn->name, alsos[alsoplace]);
 			    
 	    /*
-	     * if we found something that matches by name, and we haven't be
-	     * there, then process it.
+	     * if we found something that matches by name, and we haven't be there, then
+	     * process it.
 	     */
 	    if(sl1 && !sl1->beenhere)
 	    {
@@ -847,8 +796,8 @@ static int load_conn (struct starter_config *cfg
 		conn->strings[KSF_ALSO]=NULL;
 		sl1->beenhere = TRUE;
 
-		/* translate things, but do not replace earlier settings!*/
-		err += translate_conn(conn, sl1, k_set, perr);
+		/* translate things, but do not replace earlier settings */
+		err += translate_conn(conn, sl1, FALSE, perr);
 
 		if(conn->strings[KSF_ALSO])
 		{
@@ -957,15 +906,11 @@ static int load_conn (struct starter_config *cfg
 		    conn->options[KBF_AUTHBY]);
 #endif
     }
-
-
     
     KW_POLICY_FLAG(KBF_REKEY, POLICY_DONT_REKEY);
 
     KW_POLICY_FLAG(KBF_AGGRMODE, POLICY_AGGRESSIVE);
 
-    KW_POLICY_FLAG(KBF_MODECONFIGPULL, POLICY_MODECFG_PULL);
-
     if(conn->strings_set[KSF_ESP]) {
 	conn->esp = xstrdup(conn->strings[KSF_ESP]);
     }
@@ -974,39 +919,13 @@ static int load_conn (struct starter_config *cfg
 	conn->ike = xstrdup(conn->strings[KSF_IKE]);
     }
 
-    if(conn->strings_set[KSF_CONNALIAS]) {
-	conn->connalias = xstrdup(conn->strings[KSF_CONNALIAS]);
-    }
-
     if(conn->options_set[KBF_PHASE2]) {
 	conn->policy &= ~(POLICY_AUTHENTICATE|POLICY_ENCRYPT);
 	conn->policy |= conn->options[KBF_PHASE2];
     }
 
-    if(conn->options_set[KBF_IKEv2]) {
-	switch(conn->options[KBF_IKEv2]) {
-	case fo_never:
-	    conn->policy &= ~POLICY_IKEV2_ALLOW;
-	    break;
-	    
-	case fo_permit:
-	    /* this is the default for now */
-	    conn->policy |= POLICY_IKEV2_ALLOW;
-	    break;
-	    
-	case fo_propose:
-	    conn->policy |= POLICY_IKEV2_ALLOW|POLICY_IKEV2_PROPOSE;
-	    break;
-	    
-	case fo_insist:
-	    conn->policy |= POLICY_IKEV1_DISABLE;
-	    conn->policy |= POLICY_IKEV2_ALLOW|POLICY_IKEV2_PROPOSE;
-	    break;
-	}
-    }
-
-    err += validate_end(conn, &conn->left,  TRUE,  resolvip, perr);
-    err += validate_end(conn, &conn->right, FALSE, resolvip, perr);
+    err += validate_end(conn, &conn->left,  TRUE, perr);
+    err += validate_end(conn, &conn->right, FALSE,perr);
 
     if(conn->options_set[KBF_AUTO]) {
 	conn->desired_state = conn->options[KBF_AUTO];
@@ -1079,8 +998,6 @@ struct starter_conn *alloc_add_conn(struct starter_config *cfg, char *name, err_
     conn->name = xstrdup(name);
     conn->desired_state = STARTUP_NO;
     conn->state = STATE_FAILED;
-
-    TAILQ_INIT(&conn->comments);
     
     TAILQ_INSERT_TAIL(&cfg->conns, conn, link);
     return conn;
@@ -1090,8 +1007,6 @@ int init_load_conn(struct starter_config *cfg
 		   , struct config_parsed *cfgp
 		   , struct section_list *sconn
 		   , bool alsoprocessing
-		   , bool defaultconn
-		   , bool resolvip
 		   , err_t *perr)
 {
     int connerr;
@@ -1103,13 +1018,8 @@ int init_load_conn(struct starter_config *cfg
 	return -1;
     }
     
-    connerr = load_conn (cfg, conn, cfgp, sconn, TRUE,
-			 defaultconn, resolvip, perr);
+    connerr = load_conn (cfg, conn, cfgp, sconn, TRUE, perr);
 		
-    if(connerr != 0) {
-	starter_log(LOG_LEVEL_INFO, "while loading '%s': %s\n",
-		    sconn->name, *perr);
-    }
     if(connerr == 0)
     {
 	conn->state = STATE_LOADED;
@@ -1118,10 +1028,7 @@ int init_load_conn(struct starter_config *cfg
 }
 
 
-struct starter_config *confread_load(const char *file
-				     , err_t *perr
-				     , bool resolvip
-				     , char *ctlbase)
+struct starter_config *confread_load(const char *file, err_t *perr, char *ctlbase)
 {
 	struct starter_config *cfg = NULL;
 	struct config_parsed *cfgp;
@@ -1167,18 +1074,12 @@ struct starter_config *confread_load(const char *file
 	{
 		if (strcmp(sconn->name,"%default")==0) {
 			starter_log(LOG_LEVEL_DEBUG, "Loading default conn");
-			err += load_conn (cfg, &cfg->conn_default,
-					  cfgp, sconn, FALSE,
-					  /*default conn*/TRUE,
-					  resolvip, perr);
+			err += load_conn (cfg, &cfg->conn_default, cfgp, sconn, FALSE, perr);
 		}
 
 		if (strcmp(sconn->name,"%oedefault")==0) {
 			starter_log(LOG_LEVEL_DEBUG, "Loading oedefault conn");
-			err += load_conn (cfg, &cfg->conn_oedefault,
-					  cfgp, sconn, FALSE,
-					  /*default conn*/TRUE,
-					  resolvip, perr);
+			err += load_conn (cfg, &cfg->conn_oedefault, cfgp, sconn, FALSE, perr);
 			if(err == 0) {
 			    cfg->got_oedefault=TRUE;
 			}
@@ -1193,8 +1094,7 @@ struct starter_config *confread_load(const char *file
 		if (strcmp(sconn->name,"%default")==0) continue;
 		if (strcmp(sconn->name,"%oedefault")==0) continue;
 
-		connerr = init_load_conn(cfg, cfgp, sconn, TRUE, FALSE,
-					 resolvip, perr);
+		connerr = init_load_conn(cfg,cfgp,sconn,TRUE,perr);
 
 		if(connerr == -1) {
 		    parser_free_conf(cfgp);
@@ -1238,15 +1138,14 @@ static void confread_free_conn(struct starter_conn *conn)
 	    FREE_STR(conn->strings[i]);
 	}
 
-	FREE_STR(conn->connalias);
-	FREE_STR(conn->name);
-
 #ifdef ALG_PATCH
 	FREE_STR(conn->esp);
 	FREE_STR(conn->ike);
 #endif
+#ifdef VIRTUAL_IP
 	FREE_STR(conn->left.virt);
 	FREE_STR(conn->right.virt);
+#endif
 }
 
 void confread_free(struct starter_config *cfg)
@@ -1254,7 +1153,9 @@ void confread_free(struct starter_config *cfg)
     int i;
 	struct starter_conn *conn, *c;
 	FREE_LST(cfg->setup.interfaces);
+#ifdef VIRTUAL_IP
 	FREE_STR(cfg->setup.virtual_private);
+#endif
 	for(i=0 ;i<KSF_MAX; i++)
 	{
 	    FREE_STR(cfg->setup.strings[i]);
