@@ -1,6 +1,6 @@
 /*
- * Openswan config file parser (keywords.c)
- * Copyright (C) 2003-2006 Michael Richardson <mcr@xelerance.com>
+ * FreeS/WAN config file parser (keywords.c)
+ * Copyright (C) 2003 Michael Richardson <mcr@sandelman.ottawa.on.ca>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -12,6 +12,7 @@
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
  *
+ * RCSID $Id: keywords.c,v 1.12 2004/12/02 16:26:02 ken Exp $
  */
 
 #include <sys/queue.h>
@@ -51,26 +52,12 @@ struct keyword_enum_values kw_failureshunt_list=
  * Values for keyexchange=
  */
 struct keyword_enum_value kw_keyexchange_values[]={
-    { "ike",  KE_IKE },
+    { "ike",  KH_IKE },
 };
 
 struct keyword_enum_values kw_keyexchange_list=
     { kw_keyexchange_values, sizeof(kw_keyexchange_values)/sizeof(struct keyword_enum_value)};
 
-/*
- * Values for Four-State options, such as ikev2
- */
-struct keyword_enum_value kw_fourvalued_values[]={
-    { "never",     fo_never  },
-    { "permit",    fo_permit },
-    { "propose",   fo_propose},
-    { "insist",    fo_insist },
-    { "yes",       fo_propose},
-    { "always",    fo_insist },
-    { "no",        fo_never  }
-};
-struct keyword_enum_values kw_fourvalued_list=
-{ kw_fourvalued_values, sizeof(kw_fourvalued_values)/sizeof(struct keyword_enum_value)};
 
 /*
  * Values for authby={rsasig, secret}
@@ -105,7 +92,6 @@ struct keyword_enum_value kw_auto_values[]={
     { "add",    STARTUP_ADD },
     { "route",  STARTUP_ROUTE },
     { "start",  STARTUP_START },
-    { "up",     STARTUP_START }, /* alias */
 };
 
 struct keyword_enum_values kw_auto_list=
@@ -162,6 +148,23 @@ struct keyword_enum_values kw_proto_stack=
     { kw_proto_stack_list, sizeof(kw_proto_stack_list)/sizeof(struct keyword_enum_value)};
 
 
+#if 0
+/*
+ * Values for acceleration={never,ipsec,pk,available}
+ *
+ * For now, these are in fact just a string.
+ */
+struct keyword_enum_value kw_accel_options_list[]={
+    { "never",        XCEL_NONE },
+    { "ipsec",        XCEL_IPSEC },
+    { "pk",           XCEL_PK }, 
+    { "available",    XCEL_IPSEC|XCEL_PK},
+};
+
+struct keyword_enum_values kw_accel_options=
+    { kw_accel_options, sizeof(kw_accel_options)/sizeof(struct keyword_enum_value)};
+#endif
+
 /*
  * Values for right= and left=
  */
@@ -181,13 +184,11 @@ struct keyword_enum_value kw_plutodebug_values[]={
     { "klips",    DBG_KLIPS },
     { "dns",      DBG_DNS },
     { "oppo",     DBG_OPPO },
-    { "oppoinfo",    DBG_OPPOINFO },
     { "controlmore", DBG_CONTROLMORE },
     { "private",  DBG_PRIVATE },
     { "x509",     DBG_X509 },
     { "dpd",      DBG_DPD }, 
     { "pfkey",    DBG_PFKEY }, 
-    { "nattraversal", DBG_NATT },
 
     { "impair-delay-adns-key-answer", IMPAIR_DELAY_ADNS_KEY_ANSWER },
     { "impair-delay-adns-txt-answer", IMPAIR_DELAY_ADNS_TXT_ANSWER },
@@ -217,9 +218,6 @@ struct keyword_enum_value kw_klipsdebug_values[]={
     { "tunnel",   LELEM(KDF_TUNNEL) },
     { "pfkey",    LELEM(KDF_PFKEY) },
     { "comp",     LELEM(KDF_COMP) },
-    { "nat-traversal", LELEM(KDF_NATT) },
-    { "nattraversal", LELEM(KDF_NATT) },
-    { "natt",     LELEM(KDF_NATT) },
 };
 
 struct keyword_enum_values kw_klipsdebug_list=
@@ -236,21 +234,6 @@ struct keyword_enum_value kw_phase2types_values[]={
 struct keyword_enum_values kw_phase2types_list=
     { kw_phase2types_values, sizeof(kw_phase2types_values)/sizeof(struct keyword_enum_value)};
 
-/*
- * Values for {left/right}sendcert={never,sendifasked,always,forcedtype}
- */
-struct keyword_enum_value kw_sendcert_values[]={
-    { "never",        cert_neversend },
-    { "sendifasked",  cert_sendifasked }, 
-    { "alwayssend",   cert_alwayssend },
-    { "always",       cert_alwayssend },
-    { "forcedtype",   cert_forcedtype },
-};
-
-struct keyword_enum_values kw_sendcert_list=
-    { kw_sendcert_values, sizeof(kw_sendcert_values)/sizeof(struct keyword_enum_value)};
-
-/* MASTER KEYWORD LIST */
 struct keyword_def ipsec_conf_keywords_v2[]={
     {"interfaces",     kv_config, kt_string,    KSF_INTERFACES,NOT_ENUM},
     {"forwardcontrol", kv_config, kt_bool,      KBF_FORWARDCONTROL,NOT_ENUM},
@@ -281,40 +264,37 @@ struct keyword_def ipsec_conf_keywords_v2[]={
     {"nat_traversal", kv_config,kt_bool,        KBF_NATTRAVERSAL, NOT_ENUM},
 #endif
     {"protostack",     kv_config, kt_string,    KSF_PROTOSTACK, &kw_proto_stack},
-    {"nhelpers",kv_config,kt_number, KBF_NHELPERS, NOT_ENUM},
+    {"nhelpers",       kv_config, kt_number, KBF_NHELPERS, NOT_ENUM},
+    {"acceleration",   kv_config, kt_string, KSF_ACCELERATION, NOT_ENUM /*&kw_accel_options */},
+    {"accelerator",    kv_config, kt_string, KSF_ACCELERATION, NOT_ENUM /*&kw_accel_options */},
 
     /* this is "left=" and "right=" */
     {"",               kv_conn|kv_leftright, kt_loose_enum, KSCF_IP, &kw_host_list},  
 
     {"ike",            kv_conn|kv_auto, kt_string, KSF_IKE,NOT_ENUM},
 
-    {"subnet",         kv_conn|kv_auto|kv_leftright|kv_processed, kt_subnet, KSCF_SUBNET,NOT_ENUM}, 
-    {"subnets",        kv_conn|kv_auto|kv_leftright, kt_appendlist, KSCF_SUBNETS,NOT_ENUM}, 
-    {"sourceip",       kv_conn|kv_auto|kv_leftright, kt_ipaddr, KSCF_SOURCEIP,NOT_ENUM}, 
-    {"nexthop",        kv_conn|kv_auto|kv_leftright, kt_ipaddr, KSCF_NEXTHOP,NOT_ENUM},
-    {"firewall",       kv_conn|kv_auto|kv_leftright, kt_bool,   KNCF_FIREWALL,NOT_ENUM},
-    {"updown",         kv_conn|kv_auto|kv_leftright, kt_filename, KSCF_UPDOWN,NOT_ENUM},
-    {"id",             kv_conn|kv_auto|kv_leftright, kt_idtype, KSCF_ID,NOT_ENUM},
-    {"rsasigkey",      kv_conn|kv_auto|kv_leftright, kt_rsakey, KSCF_RSAKEY1, &kw_rsasigkey_list},
-    {"rsasigkey2",     kv_conn|kv_auto|kv_leftright, kt_rsakey, KSCF_RSAKEY2, &kw_rsasigkey_list},
+    {"subnet",         kv_conn|kv_leftright|kv_processed, kt_subnet, KSCF_SUBNET,NOT_ENUM}, 
+    {"sourceip",       kv_conn|kv_leftright, kt_ipaddr, KSCF_SOURCEIP,NOT_ENUM}, 
+    {"nexthop",        kv_conn|kv_leftright, kt_ipaddr, KSCF_NEXTHOP,NOT_ENUM},
+    {"firewall",       kv_conn|kv_leftright, kt_bool,   KNCF_FIREWALL,NOT_ENUM},
+    {"updown",         kv_conn|kv_leftright, kt_filename, KSCF_UPDOWN,NOT_ENUM},
+    {"id",             kv_conn|kv_leftright, kt_idtype, KSCF_ID,NOT_ENUM},
+    {"rsasigkey",      kv_conn|kv_leftright, kt_rsakey, KSCF_RSAKEY1, &kw_rsasigkey_list},
+    {"rsasigkey2",     kv_conn|kv_leftright, kt_rsakey, KSCF_RSAKEY2, &kw_rsasigkey_list},
     {"spibase",        kv_conn|kv_auto|kv_leftright, kt_number, KNCF_SPIBASE,NOT_ENUM},
-    {"cert",           kv_conn|kv_auto|kv_leftright, kt_filename, KSCF_CERT,NOT_ENUM},
-    {"sendcert",       kv_conn|kv_auto|kv_leftright, kt_enum,   KNCF_SENDCERT, &kw_sendcert_list},
-    {"ca",             kv_conn|kv_auto|kv_leftright, kt_string, KSCF_CA,NOT_ENUM},
+    {"cert",           kv_conn|kv_leftright, kt_filename, KSCF_CERT,NOT_ENUM},
+    {"ca",             kv_conn|kv_leftright, kt_string, KSCF_CA,NOT_ENUM},
 
-    /* these are conn statements which are not left/right */
-    {"auto",           kv_conn|kv_duplicateok, kt_enum,   KBF_AUTO,        &kw_auto_list},
+    {"auto",           kv_conn,         kt_enum,   KBF_AUTO,        &kw_auto_list},
     {"also",           kv_conn,         kt_appendstring, KSF_ALSO,NOT_ENUM},
     {"alsoflip",       kv_conn,         kt_string, KSF_ALSOFLIP,NOT_ENUM},
     {"type",           kv_conn,         kt_enum,   KBF_TYPE,        &kw_type_list},
     {"authby",         kv_conn|kv_auto, kt_enum,   KBF_AUTHBY,     &kw_authby_list},
     {"keyexchange",    kv_conn|kv_auto, kt_enum,   KBF_KEYEXCHANGE, &kw_keyexchange_list},
-    {"ikev2",          kv_conn|kv_auto|kv_processed,kt_enum,KBF_IKEv2,&kw_fourvalued_list},
     {"pfs",            kv_conn|kv_auto, kt_bool,   KBF_PFS,          NOT_ENUM},
     {"keylife",        kv_conn|kv_auto|kv_alias, kt_time,   KBF_SALIFETIME,NOT_ENUM},
     {"lifetime",       kv_conn|kv_auto|kv_alias, kt_time,   KBF_SALIFETIME,NOT_ENUM},
     {"salifetime",     kv_conn|kv_auto, kt_time,   KBF_SALIFETIME,NOT_ENUM},
-    {"forceencaps",    kv_conn|kv_auto, kt_bool,   KBF_FORCEENCAP, NOT_ENUM},
     {"rekey",          kv_conn|kv_auto, kt_bool,   KBF_REKEY, NOT_ENUM},
     {"rekeymargin",    kv_conn|kv_auto, kt_time,   KBF_REKEYMARGIN,NOT_ENUM},
     {"rekeyfuzz",      kv_conn|kv_auto, kt_time,   KBF_REKEYFUZZ,NOT_ENUM},
@@ -322,7 +302,6 @@ struct keyword_def ipsec_conf_keywords_v2[]={
     {"ikelifetime",    kv_conn|kv_auto, kt_time,   KBF_IKELIFETIME,NOT_ENUM},
     {"disablearrivalcheck", kv_conn|kv_auto, kt_invertbool, KBF_ARRIVALCHECK,NOT_ENUM},
     {"failureshunt",   kv_conn|kv_auto, kt_enum,   KBF_FAILURESHUNT, &kw_failureshunt_list},
-    {"connalias",      kv_conn|kv_processed|kv_auto|kv_manual, kt_appendstring,   KSF_CONNALIAS, NOT_ENUM},
 
     /* attributes of the phase2 policy */
     {"phase2alg",      kv_conn|kv_auto|kv_manual,  kt_string, KSF_ESP,NOT_ENUM},
@@ -343,17 +322,13 @@ struct keyword_def ipsec_conf_keywords_v2[]={
 
     /* aggr/xauth/modeconfig */ 
     {"aggrmode",    kv_conn|kv_auto, kt_invertbool,      KBF_AGGRMODE,NOT_ENUM},
-    {"xauthserver", kv_conn|kv_auto|kv_leftright, kt_bool, KNCF_XAUTHSERVER,  NOT_ENUM},
-    {"xauthclient", kv_conn|kv_auto|kv_leftright, kt_bool, KNCF_XAUTHCLIENT, NOT_ENUM},
-    {"xauthname",   kv_conn|kv_auto|kv_leftright, kt_string, KSCF_XAUTHUSERNAME, NOT_ENUM},
-    {"modecfgserver", kv_conn|kv_auto|kv_leftright, kt_bool, KNCF_MODECONFIGSERVER, NOT_ENUM},
-    {"modecfgclient", kv_conn|kv_auto|kv_leftright, kt_bool, KNCF_MODECONFIGCLIENT, NOT_ENUM},
-    {"xauthusername", kv_conn|kv_auto|kv_leftright, kt_string, KSCF_XAUTHUSERNAME, NOT_ENUM},
+    {"xauthserver", kv_conn|kv_leftright, kt_invertbool, KNCF_XAUTHSERVER , NOT_ENUM},
+    {"xauthclient", kv_conn|kv_leftright, kt_invertbool, KNCF_XAUTHCLIENT, NOT_ENUM},
+    {"modecfgserver", kv_conn|kv_leftright, kt_invertbool, KNCF_MODECONFIGSERVER, NOT_ENUM},
+    {"modecfgclient", kv_conn|kv_leftright, kt_invertbool, KNCF_MODECONFIGCLIENT, NOT_ENUM},
     {"modecfgpull", kv_conn|kv_auto, kt_invertbool, KBF_MODECONFIGPULL , NOT_ENUM},
-    {"modecfgdns1", kv_conn|kv_auto|kv_leftright, kt_ipaddr, KSCF_MODECFGDNS1,NOT_ENUM},
-    {"modecfgdns2", kv_conn|kv_auto|kv_leftright, kt_ipaddr, KSCF_MODECFGDNS2,NOT_ENUM},
-    {"modecfgwins1", kv_conn|kv_auto|kv_leftright, kt_ipaddr, KSCF_MODECFGWINS1,NOT_ENUM},
-    {"modecfgwins2", kv_conn|kv_auto|kv_leftright, kt_ipaddr, KSCF_MODECFGWINS2,NOT_ENUM},
+
+
     /* things for manual keying only */
     {"spi",            kv_conn|kv_leftright|kv_manual, kt_number, KNCF_SPI,NOT_ENUM},
     {"espenckey",      kv_conn|kv_leftright|kv_manual, kt_bitstring, KSCF_ESPENCKEY,NOT_ENUM},
@@ -362,11 +337,6 @@ struct keyword_def ipsec_conf_keywords_v2[]={
     {NULL, 0, 0, 0, NOT_ENUM}
 };
 
-/* distinguished keyword */
-struct keyword_def ipsec_conf_keyword_comment=
-{"x-comment",      kv_conn,   kt_comment, 0, NOT_ENUM};
-
-
 const int ipsec_conf_keywords_v2_count = sizeof(ipsec_conf_keywords_v2)/sizeof(struct keyword_def);
 
 /*
@@ -385,7 +355,6 @@ int parser_find_keyword(const char *s, YYSTYPE *lval)
 
     keyleft=FALSE;
     k = ipsec_conf_keywords_v2;
-
     while(k->keyname != NULL) {
 	if(strcasecmp(s, k->keyname) == 0)
 	{
@@ -411,15 +380,7 @@ int parser_find_keyword(const char *s, YYSTYPE *lval)
 	k++;
     }
 
-    lval->s = NULL;
     /* if we found nothing */
-    if(k->keyname == NULL && (s[0]=='x' || s[0]=='X') && s[1]=='-')
-    {
-	k = &ipsec_conf_keyword_comment;
-	lval->k.string = strdup(s);
-    }
-
-    /* if we still found nothing */
     if(k->keyname == NULL) {
 	lval->s = strdup(s);
 	return STRING;
@@ -433,9 +394,6 @@ int parser_find_keyword(const char *s, YYSTYPE *lval)
     case kt_time:
 	keywordtype = TIMEWORD;
 	break;
-    case kt_comment:
-	keywordtype = COMMENT;
-	break;
     case kt_bool:
     case kt_invertbool:
 	keywordtype = BOOLWORD;
