@@ -38,8 +38,8 @@
 
 #define __NO_VERSION__
 #ifndef AUTOCONF_INCLUDED
-#include <linux/config.h>
-#endif	/* for CONFIG_IP_FORWARD */
+#include <linux/config.h>	/* for CONFIG_IP_FORWARD */
+#endif
 #ifdef CONFIG_MODULES
 #include <linux/module.h>
 #endif
