@@ -95,10 +95,60 @@ static stf_status
 aggr_inI1_outR1_tail(struct pluto_crypto_req_cont *pcrc
 		     , struct pluto_crypto_req *r);
 
+
 static void
-aggr_inI1_outR1_continue(struct pluto_crypto_req_cont *pcrc
-			 , struct pluto_crypto_req *r
-			 , err_t ugh)
+aggr_inR1_outI2_crypto_continue(struct pluto_crypto_req_cont *pcrc
+				, struct pluto_crypto_req *r
+				, err_t ugh);
+
+
+/*
+ * continuation from second calculation (the DH one)
+ *
+ */
+static void 
+aggr_inI1_outR1_continue2(struct pluto_crypto_req_cont *pcrc
+			  , struct pluto_crypto_req *r
+			  , err_t ugh)
+{
+  struct dh_continuation *dh = (struct dh_continuation *)pcrc;
+  struct msg_digest *md = dh->md;
+  struct state *const st = md->st;
+  stf_status e;
+  
+  DBG(DBG_CONTROLMORE
+      , DBG_log("aggr inI1_outR1: calculated ke+nonce+DH, sending R1"));
+  
+  /* XXX should check out ugh */
+  passert(ugh == NULL);
+  passert(cur_state == NULL);
+  passert(st != NULL);
+
+  passert(st->st_suspended_md == dh->md);
+  set_suspended(st, NULL);	/* no longer connected or suspended */
+
+  set_cur_state(st);
+  st->st_calculating = FALSE;
+
+  e = aggr_inI1_outR1_tail(pcrc, r);
+  
+  if(dh->md != NULL) {
+      complete_state_transition(&dh->md, e);
+      if(dh->md) release_md(dh->md);
+  }
+  reset_cur_state();
+}
+
+/*
+ * for aggressive mode, this is sub-optimal, since we should have
+ * had the crypto helper actually do everything, but we need to do
+ * some additional work to set that all up, so this is fine for now.
+ *
+ */
+static void
+aggr_inI1_outR1_continue1(struct pluto_crypto_req_cont *pcrc
+			  , struct pluto_crypto_req *r
+			  , err_t ugh)
 {
   struct ke_continuation *ke = (struct ke_continuation *)pcrc;
   struct msg_digest *md = ke->md;
@@ -106,7 +156,7 @@ aggr_inI1_outR1_continue(struct pluto_crypto_req_cont *pcrc
   stf_status e;
   
   DBG(DBG_CONTROLMORE
-      , DBG_log("aggr inI1_outR1: calculated ke+nonce, sending R1"));
+      , DBG_log("aggr inI1_outR1: calculated ke+nonce, calculating DH"));
   
   /* XXX should check out ugh */
   passert(ugh == NULL);
@@ -114,17 +164,40 @@ aggr_inI1_outR1_continue(struct pluto_crypto_req_cont *pcrc
   passert(st != NULL);
 
   passert(st->st_suspended_md == ke->md);
-  st->st_suspended_md = NULL;	/* no longer connected or suspended */
+  set_suspended(st, NULL);	/* no longer connected or suspended */
 
   set_cur_state(st);
   st->st_calculating = FALSE;
-  e = aggr_inI1_outR1_tail(pcrc, r);
+
+  /* unpack first calculation */
+  unpack_KE(st, r, &st->st_gr);
+
+  /* unpack nonce too */
+  unpack_nonce(&st->st_nr, r);
+
+  /* NOTE: the "r" reply will get freed by our caller */
   
-  if(ke->md != NULL) {
-      complete_state_transition(&ke->md, e);
-      release_md(ke->md);
+  /* set up second calculation */
+  {
+      struct dh_continuation *dh = alloc_thing(struct dh_continuation
+					       , "aggr outR1 DH");
+      dh->md = md;
+      set_suspended(st, md);
+      dh->dh_pcrc.pcrc_func = aggr_inI1_outR1_continue2;
+      e = start_dh_secretiv(&dh->dh_pcrc, st
+			    , st->st_import
+			    , RESPONDER
+			    , st->st_oakley.group->group);
+      
+      if(e != STF_SUSPEND) {
+	  if(dh->md != NULL) {
+	      complete_state_transition(&dh->md, e);
+	      if(dh->md) release_md(dh->md);
+	  }
+      }
+
+      reset_cur_state();
   }
-  reset_cur_state();
 }
 
 static stf_status
@@ -225,9 +298,6 @@ aggr_inI1_outR1_common(struct msg_digest *md
 
     insert_state(st);	/* needs cookies, connection, and msgid (0) */
 
-    /* copy the quirks we might have accumulated */
-    copy_quirks(&st->quirks,&md->quirks);
-
     st->st_doi = ISAKMP_DOI_IPSEC;
     st->st_situation = SIT_IDENTITY_ONLY; /* We only support this */
 
@@ -271,10 +341,10 @@ aggr_inI1_outR1_common(struct msg_digest *md
 	struct ke_continuation *ke = alloc_thing(struct ke_continuation
 						 , "outI2 KE");
 	ke->md = md;
-	st->st_suspended_md = md;
+	set_suspended(st, md);
 
 	if (!st->st_sec_in_use) {
-	    ke->ke_pcrc.pcrc_func = aggr_inI1_outR1_continue;
+	    ke->ke_pcrc.pcrc_func = aggr_inI1_outR1_continue1;
 	    return build_ke(&ke->ke_pcrc, st, st->st_oakley.group
 			    , st->st_import);
 	} else {
@@ -284,6 +354,9 @@ aggr_inI1_outR1_common(struct msg_digest *md
     }
 }
 
+
+
+
 stf_status
 aggr_inI1_outR1_psk(struct msg_digest *md)
 {
@@ -312,6 +385,8 @@ aggr_inI1_outR1_tail(struct pluto_crypto_req_cont *pcrc
      * so we have to build our md->reply and emit HDR before calling it.
      */
 
+    finish_dh_secretiv(st, r);
+
     init_pbs(&md->reply, reply_buffer, sizeof(reply_buffer), "reply packet");
 
     /* HDR out */
@@ -348,13 +423,12 @@ aggr_inI1_outR1_tail(struct pluto_crypto_req_cont *pcrc
     /************** build rest of output: KE, Nr, IDir, HASH_R/SIG_R ********/
 
     /* KE */
-    if (!ship_KE(st, r, &st->st_gr, 
-		 &md->rbody, ISAKMP_NEXT_NONCE))
+    if (!justship_KE(&st->st_gr,
+		     &md->rbody, ISAKMP_NEXT_NONCE))
 	return STF_INTERNAL_ERROR;
 
     /* Nr */
-    if (!ship_nonce(&st->st_nr, r
-		    , &md->rbody, ISAKMP_NEXT_ID, "Nr"))
+    if (!justship_nonce(&st->st_nr, &md->rbody, ISAKMP_NEXT_ID, "Nr"))
 	return STF_INTERNAL_ERROR;
 
     /* IDir out */
@@ -370,7 +444,6 @@ aggr_inI1_outR1_tail(struct pluto_crypto_req_cont *pcrc
 	close_output_pbs(&r_id_pbs);
     }
 
-    (void)perform_dh_secretiv(st, RESPONDER, st->st_oakley.group->group);
     update_iv(st);
 
 
@@ -528,15 +601,53 @@ aggr_inR1_outI2(struct msg_digest *md)
     }
 #endif
 
+    /* set up second calculation */
     {
-	stf_status stat;
-	stat = perform_dh_secretiv(st, INITIATOR, st->st_oakley.group->group);
-	if(stat != STF_OK) {
-	    return stat;
-	}
+	struct dh_continuation *dh = alloc_thing(struct dh_continuation
+						 , "aggr outR1 DH");
+	dh->md = md;
+	set_suspended(st, md);
+	dh->dh_pcrc.pcrc_func = aggr_inR1_outI2_crypto_continue;
+	return start_dh_secretiv(&dh->dh_pcrc, st
+				 , st->st_import
+				 , INITIATOR
+				 , st->st_oakley.group->group);
     }
+}
 
-    return aggr_inR1_outI2_tail(md, NULL);
+static void
+aggr_inR1_outI2_crypto_continue(struct pluto_crypto_req_cont *pcrc
+				, struct pluto_crypto_req *r
+				, err_t ugh)
+{
+  struct dh_continuation *dh = (struct dh_continuation *)pcrc;
+  struct msg_digest *md = dh->md;
+  struct state *const st = md->st;
+  stf_status e;
+  
+  DBG(DBG_CONTROLMORE
+      , DBG_log("aggr inR1_outI2: calculated DH, sending I2"));
+  
+  /* XXX should check out ugh */
+  passert(ugh == NULL);
+  passert(cur_state == NULL);
+  passert(st != NULL);
+
+  passert(st->st_suspended_md == dh->md);
+  set_suspended(st, NULL);	/* no longer connected or suspended */
+
+  set_cur_state(st);
+  st->st_calculating = FALSE;
+
+  finish_dh_secretiv(st, r);
+
+  e = aggr_inR1_outI2_tail(md, NULL);
+  
+  if(dh->md != NULL) {
+      complete_state_transition(&dh->md, e);
+      if(dh->md) release_md(dh->md);
+  }
+  reset_cur_state();
 }
 
 static void
@@ -771,7 +882,7 @@ aggr_outI1_continue(struct pluto_crypto_req_cont *pcrc
   passert(st != NULL);
 
   passert(st->st_suspended_md == ke->md);
-  st->st_suspended_md = NULL;	/* no longer connected or suspended */
+  set_suspended(st,NULL);	/* no longer connected or suspended */
 
   set_cur_state(st);
 
@@ -781,7 +892,7 @@ aggr_outI1_continue(struct pluto_crypto_req_cont *pcrc
   
   if(ke->md != NULL) {
       complete_state_transition(&ke->md, e);
-      release_md(ke->md);
+      if(ke->md) release_md(ke->md);
   }
   reset_globals();
 
@@ -797,7 +908,6 @@ aggr_outI1(int whack_sock,
 	   , enum crypto_importance importance)
 {
     struct state *st;
-    struct spd_route *sr;
 
     /* set up new state */
     cur_state = st = new_state();
@@ -814,17 +924,6 @@ aggr_outI1(int whack_sock,
 
     get_cookie(TRUE, st->st_icookie, COOKIE_SIZE, &c->spd.that.host_addr);
 
-    st->st_import = importance;
-
-    for(sr=&c->spd; sr!=NULL; sr=sr->next) {
-	if(sr->this.xauth_client) {
-	    if(sr->this.xauth_name) {
-		strncpy(st->st_xauth_username, sr->this.xauth_name, sizeof(st->st_xauth_username));
-		break;
-	    }
-	}
-    }
-
     insert_state(st);	/* needs cookies, connection, and msgid (0) */
 
     if(init_am_st_oakley(st, policy) == FALSE) {
@@ -854,20 +953,16 @@ aggr_outI1(int whack_sock,
 
 	ke->md = alloc_md();
 	ke->md->st = st;
-	st->st_suspended_md = ke->md;
+	set_suspended(st, ke->md);
 
-	if (!st->st_sec_in_use) {
-	    ke->ke_pcrc.pcrc_func = aggr_outI1_continue;
-	    e = build_ke(&ke->ke_pcrc, st, st->st_oakley.group, importance);
-	    if(e != STF_SUSPEND && e != STF_INLINE) {
-	      loglog(RC_CRYPTOFAILED, "system too busy");
-	      delete_state(st);
-	    }
-	} else {
-	    e = aggr_outI1_tail((struct pluto_crypto_req_cont *)ke
-					, NULL);
+	passert(st->st_sec_in_use == FALSE);
+	ke->ke_pcrc.pcrc_func = aggr_outI1_continue;
+	e = build_ke(&ke->ke_pcrc, st, st->st_oakley.group, importance);
+	if(e != STF_SUSPEND) {
+	    loglog(RC_CRYPTOFAILED, "system too busy");
+	    delete_state(st);
 	}
-	
+
 	reset_globals();
 
 	return e;
@@ -933,6 +1028,8 @@ aggr_outI1_tail(struct pluto_crypto_req_cont *pcrc
     if (!ship_nonce(&st->st_ni, r, &md->rbody, ISAKMP_NEXT_ID, "Ni"))
 	return STF_INTERNAL_ERROR;
 
+    DBG_log("setting sec: %d", st->st_sec_in_use);
+
     /* IDii out */
     {
 	struct isakmp_ipsec_id id_hd;
