@@ -33,6 +33,7 @@
 #include <openswan.h>
 #include <openswan/ipsec_policy.h>
 
+#include "openswan/pfkeyv2.h"
 #include "sysdep.h"
 #include "constants.h"
 #include "oswlog.h"
@@ -295,23 +296,10 @@ fmt_common_shell_out(char *buf, int blen, struct connection *c
 	secure_myid_str[IDTOA_BUF] = "",
 	secure_peerid_str[IDTOA_BUF] = "",
 	secure_peerca_str[IDTOA_BUF] = "",
-	nexthop_str[sizeof("PLUTO_NEXT_HOP='' ")+ADDRTOT_BUF],
 	secure_xauth_username_str[IDTOA_BUF] = "";
     
     ip_address ta;
     
-    nexthop_str[0]='\0';
-    if(addrbytesptr(&sr->this.host_nexthop, NULL)
-       && !isanyaddr(&sr->this.host_nexthop))
-    {
-	char *n;
-	strcpy(nexthop_str, "PLUTO_NEXT_HOP='");
-	n = nexthop_str + strlen(nexthop_str);
-	addrtot(&sr->this.host_nexthop, 0,
-		n, sizeof(nexthop_str)-strlen(nexthop_str));
-	strncat(nexthop_str, "' ", sizeof(nexthop_str));
-    }
-
     addrtot(&sr->this.host_addr, 0, me_str, sizeof(me_str));
     idtoa(&sr->this.id, myid_str, sizeof(myid_str));
     escape_metachar(myid_str, secure_myid_str, sizeof(secure_myid_str));
@@ -336,7 +324,7 @@ fmt_common_shell_out(char *buf, int blen, struct connection *c
 	strcpy(secure_xauth_username_str, "PLUTO_XAUTH_USERNAME='");
 	
 	len = strlen(secure_xauth_username_str);
-	remove_metachar((unsigned char *)st->st_xauth_username
+	remove_metachar(st->st_xauth_username
 			,secure_xauth_username_str+len
 			,sizeof(secure_xauth_username_str)-(len+2));
 	strncat(secure_xauth_username_str, "'", sizeof(secure_xauth_username_str)-1);
@@ -380,7 +368,6 @@ fmt_common_shell_out(char *buf, int blen, struct connection *c
 		    "PLUTO_VERSION='2.0' "  /* change VERSION when interface spec changes */
 		    "PLUTO_CONNECTION='%s' "
 		    "PLUTO_INTERFACE='%s' "
-		    "%s"      /* possible PLUTO_NEXT_HOP */
 		    "PLUTO_ME='%s' "
 		    "PLUTO_MY_ID='%s' "
 		    "PLUTO_MY_CLIENT='%s' "
@@ -402,7 +389,6 @@ fmt_common_shell_out(char *buf, int blen, struct connection *c
 		    "%s "           /* PLUTO_MY_SRCIP */
 		    , c->name
 		    , c->interface->ip_dev->id_vname
-		    , nexthop_str
 		    , me_str
 		    , secure_myid_str
 		    , myclient_str
@@ -504,14 +490,12 @@ could_route(struct connection *c)
         return route_impossible;
     }
 
-#if 0
     /* if we don't know nexthop, we cannot route */
     if (isanyaddr(&c->spd.this.host_nexthop))
     {
         loglog(RC_ROUTE, "cannot route connection without knowing our nexthop");
         return route_impossible;
     }
-#endif
 
     /* if routing would affect IKE messages, reject */
     if (kern_interface != NO_KERNEL
@@ -804,7 +788,7 @@ raw_eroute(const ip_address *this_host
            , ipsec_spi_t spi
            , unsigned int proto
            , unsigned int transport_proto
-           , unsigned int satype
+           , enum eroute_type esatype
            , const struct pfkey_proto_info *proto_info
            , time_t use_lifetime
            , enum pluto_sadb_operations op
@@ -813,7 +797,6 @@ raw_eroute(const ip_address *this_host
     char text_said[SATOT_BUF];
     int sport = ntohs(portof(&this_client->addr));
     int dport = ntohs(portof(&that_client->addr));
-    bool result;
 
     set_text_said(text_said, that_host, spi, proto);
 
@@ -829,18 +812,12 @@ raw_eroute(const ip_address *this_host
                     , text_said);
         });
 
-    result = kernel_ops->raw_eroute(this_host, this_client
+    return kernel_ops->raw_eroute(this_host, this_client
                                   , that_host, that_client
                                   , spi, proto
                                   , transport_proto
-                                  , satype, proto_info
+                                  , esatype, proto_info
                                   , use_lifetime, op, text_said);
-    
-    if(result == FALSE || DBGP(DBG_CONTROL|DBG_KLIPS)) {
-	   DBG_log("raw_eroute result=%u\n", result);
-    }
-
-    return result;
 }
 
 /* test to see if %hold remains */
@@ -902,12 +879,11 @@ replace_bare_shunt(const ip_address *src, const ip_address *dst
                 /* is there already a broad host-to-host bare shunt? */
                 if (bs_pp == NULL)
                     {
-			DBG(DBG_KLIPS, DBG_log("replacing broad host-to-host bare shunt"));
                         if (raw_eroute(null_host, &this_broad_client
                                        , null_host, &that_broad_client
                                        , htonl(shunt_spi), SA_INT
                                        , transport_proto
-                                       , SADB_X_SATYPE_INT, null_proto_info
+                                       , ET_INT, null_proto_info
                                        , SHUNT_PATIENCE, ERO_REPLACE, why))
                             {
                                 struct bare_shunt *bs = alloc_thing(struct bare_shunt, "bare shunt");
@@ -930,12 +906,11 @@ replace_bare_shunt(const ip_address *src, const ip_address *dst
                 shunt_spi = SPI_HOLD;
             }
         
-	DBG(DBG_KLIPS, DBG_log("adding specific host-to-host bare shunt"));
         if (raw_eroute(null_host, &this_client, null_host, &that_client
                        , htonl(shunt_spi)
                        , SA_INT
                        , transport_proto
-                       , SADB_X_SATYPE_INT, null_proto_info
+                       , ET_INT, null_proto_info
                        , SHUNT_PATIENCE, ERO_ADD, why))
             {
                 struct bare_shunt **bs_pp = bare_shunt_ptr(&this_client, &that_client
@@ -952,13 +927,12 @@ replace_bare_shunt(const ip_address *src, const ip_address *dst
             }
     }
     else {
-        unsigned int op = repl ? ERO_REPLACE : ERO_DELETE;
+        unsigned int op = repl? ERO_REPLACE : ERO_DELETE;
         
-	DBG(DBG_KLIPS, DBG_log("%s specific host-to-host bare shunt", repl ? "replacing" : "removing"));
         if (raw_eroute(null_host, &this_client, null_host, &that_client
                        , htonl(shunt_spi), SA_INT
                        , 0 /* transport_proto */
-                       , SADB_X_SATYPE_INT, null_proto_info
+                       , ET_INT, null_proto_info
                        , SHUNT_PATIENCE, op, why))
             {
                 struct bare_shunt **bs_pp = bare_shunt_ptr(&this_client
@@ -996,7 +970,7 @@ replace_bare_shunt(const ip_address *src, const ip_address *dst
 
 bool eroute_connection(struct spd_route *sr
 		       , ipsec_spi_t spi, unsigned int proto
-		       , unsigned int satype
+		       , enum eroute_type esatype
 		       , const struct pfkey_proto_info *proto_info
 		       , unsigned int op, const char *opname)
 {
@@ -1015,7 +989,7 @@ bool eroute_connection(struct spd_route *sr
                       , spi
                       , proto
                       , sr->this.protocol
-                      , satype
+                      , esatype
                       , proto_info, 0, op, buf2);
 }
 
@@ -1074,11 +1048,11 @@ assign_hold(struct connection *c USED_BY_DEBUG
         if (rn != ro)
         {
             if (erouted(ro)
-            ? !eroute_connection(sr, htonl(SPI_HOLD), SA_INT, SADB_X_SATYPE_INT
+            ? !eroute_connection(sr, htonl(SPI_HOLD), SA_INT, ET_INT
                                  , null_proto_info
                                  , ERO_REPLACE
                                  , "replace %trap with broad %hold")
-            : !eroute_connection(sr, htonl(SPI_HOLD), SA_INT, SADB_X_SATYPE_INT
+            : !eroute_connection(sr, htonl(SPI_HOLD), SA_INT, ET_INT
                                  , null_proto_info
                 , ERO_ADD, "add broad %hold"))
             {
@@ -1158,7 +1132,8 @@ setup_half_ipsec_sa(struct state *st, bool inbound)
     ip_subnet src, dst;
     ip_subnet src_client, dst_client;
     ipsec_spi_t inner_spi = 0;
-    unsigned int proto = 0, satype = 0;
+    unsigned int proto = 0;
+    enum eroute_type esatype = 0;
     bool replace;
     bool outgoing_ref_set = FALSE;
     bool incoming_ref_set = FALSE;
@@ -1209,7 +1184,7 @@ setup_half_ipsec_sa(struct state *st, bool inbound)
     {
         inner_spi = 256;
         proto = SA_IPIP;
-        satype = SADB_SATYPE_UNSPEC;
+        esatype = ET_IPIP;
     }
     else if (encapsulation == ENCAPSULATION_MODE_TUNNEL)
     {
@@ -1237,13 +1212,14 @@ setup_half_ipsec_sa(struct state *st, bool inbound)
         set_text_said(text_said
             , &c->spd.that.host_addr, ipip_spi, SA_IPIP);
 
+	said_next->proto = IPPROTO_IPIP;
         said_next->src = &src.addr;
         said_next->dst = &dst.addr;
         said_next->src_client = &src_client;
         said_next->dst_client = &dst_client;
         said_next->spi = ipip_spi;
-        said_next->satype = SADB_X_SATYPE_IPIP;
-        said_next->text_said = text_said;
+        said_next->esatype = ET_IPIP;
+        said_next->text_said = clone_str(text_said, "said");
 	said_next->sa_lifetime = c->sa_ipsec_life_seconds;
 
 	if(inbound) {
@@ -1285,7 +1261,7 @@ setup_half_ipsec_sa(struct state *st, bool inbound)
 
         inner_spi = ipip_spi;
         proto = SA_IPIP;
-        satype = SADB_X_SATYPE_IPIP;
+        esatype = ET_IPIP;
     }
 
     /* set up IPCOMP SA, if any */
@@ -1309,16 +1285,17 @@ setup_half_ipsec_sa(struct state *st, bool inbound)
 
         set_text_said(text_said, &dst.addr, ipcomp_spi, SA_COMP);
 
+	said_next->proto = IPPROTO_COMP;
         said_next->src = &src.addr;
         said_next->dst = &dst.addr;
         said_next->src_client = &src_client;
         said_next->dst_client = &dst_client;
         said_next->spi = ipcomp_spi;
-        said_next->satype = SADB_X_SATYPE_COMP;
+        said_next->esatype = ET_IPCOMP;
         said_next->encalg = compalg;
         said_next->encapsulation = encapsulation;
         said_next->reqid = c->spd.reqid + 2;
-        said_next->text_said = text_said;
+        said_next->text_said = clone_str(text_said, "said");
 	said_next->sa_lifetime = c->sa_ipsec_life_seconds;
 
 	if(inbound) {
@@ -1515,12 +1492,13 @@ setup_half_ipsec_sa(struct state *st, bool inbound)
 
         set_text_said(text_said, &dst.addr, esp_spi, SA_ESP);
 
+	said_next->proto = IPPROTO_ESP;
         said_next->src = &src.addr;
         said_next->dst = &dst.addr;
         said_next->src_client = &src_client;
         said_next->dst_client = &dst_client;
         said_next->spi = esp_spi;
-        said_next->satype = SADB_SATYPE_ESP;
+        said_next->esatype = ET_ESP;
         said_next->replay_window = kernel_ops->replay_window;
         said_next->authalg = ei->authalg;
         said_next->authkeylen = ei->authkeylen;
@@ -1539,7 +1517,7 @@ setup_half_ipsec_sa(struct state *st, bool inbound)
         said_next->natt_type = natt_type;
         said_next->natt_oa = &natt_oa;
 #endif  
-        said_next->text_said = text_said;
+        said_next->text_said = clone_str(text_said, "said");
 	said_next->sa_lifetime = c->sa_ipsec_life_seconds;
 
 #ifdef DIVULGE_KEYS
@@ -1611,19 +1589,20 @@ setup_half_ipsec_sa(struct state *st, bool inbound)
 
         set_text_said(text_said, &dst.addr, ah_spi, SA_AH);
 
+	said_next->proto = IPPROTO_AH;
         said_next->src = &src.addr;
         said_next->dst = &dst.addr;
         said_next->src_client = &src_client;
         said_next->dst_client = &dst_client;
         said_next->spi = ah_spi;
-        said_next->satype = SADB_SATYPE_AH;
+        said_next->esatype = ET_AH;
         said_next->replay_window = kernel_ops->replay_window;
         said_next->authalg = authalg;
         said_next->authkeylen = st->st_ah.keymat_len;
         said_next->authkey = ah_dst_keymat;
         said_next->encapsulation = encapsulation;
         said_next->reqid = c->spd.reqid;
-        said_next->text_said = text_said;
+        said_next->text_said = clone_str(text_said, "said");
 	said_next->sa_lifetime = c->sa_ipsec_life_seconds;
 
 #ifdef DIVULGE_KEYS
@@ -1720,7 +1699,8 @@ setup_half_ipsec_sa(struct state *st, bool inbound)
                     proto_info[i].encapsulation = ENCAPSULATION_MODE_TRANSPORT;
                 }
             }
-            
+
+	    setup_client_ports(&c->spd);
             /* MCR - should be passed a spd_eroute structure here */
             (void) raw_eroute(&c->spd.that.host_addr   /* this_host */
 			      , &c->spd.that.client    /* this_client */
@@ -1729,7 +1709,7 @@ setup_half_ipsec_sa(struct state *st, bool inbound)
                               , inner_spi              /* spi */
 			      , proto                  /* proto */
                               , c->spd.this.protocol   /* transport_proto */
-                              , satype                 /* satype */
+                              , esatype                 /* satype */
                               , proto_info             /* " */
 			      , 0                      /* lifetime */
                               , ERO_ADD_INBOUND        /* op */
@@ -1749,21 +1729,10 @@ setup_half_ipsec_sa(struct state *st, bool inbound)
          */
         for (s = said; s < said_next-1; s++)
         {
-            char
-                text_said0[SATOT_BUF],
-                text_said1[SATOT_BUF];
-
             /* group s[1] and s[0], in that order */
-            
-            set_text_said(text_said0, s[0].dst, s[0].spi, s[0].proto);
-            set_text_said(text_said1, s[1].dst, s[1].spi, s[1].proto);
-            
             DBG(DBG_KLIPS, DBG_log("grouping %s (ref=%u) and %s (ref=%u)"
-				   , text_said1, s[0].ref
-				   , text_said0, s[1].ref));
-            
-            s[0].text_said = text_said0;
-            s[1].text_said = text_said1;
+				   , s[1].text_said, s[0].ref
+				   , s[0].text_said, s[1].ref));
             
             if (!kernel_ops->grp_sa(s + 1, s))
                 goto fail;
@@ -1782,6 +1751,11 @@ setup_half_ipsec_sa(struct state *st, bool inbound)
 	goto fail;
     }
 #endif
+
+    while (said_next-- != said) {
+	pfree(said_next->text_said);
+    }
+    
     return TRUE;
 
 fail:
@@ -1792,6 +1766,7 @@ fail:
 		(void) del_spi(said_next->spi, said_next->proto
 			       , &src.addr, said_next->dst);
 	    }
+	    pfree(said_next->text_said);
 	}
         return FALSE;
     }
@@ -1819,12 +1794,13 @@ teardown_half_ipsec_sa(struct state *st, bool inbound)
     if (kernel_ops->inbound_eroute && inbound
         && c->spd.eroute_owner == SOS_NOBODY)
     {
+	setup_client_ports(&c->spd);
         (void) raw_eroute(&c->spd.that.host_addr, &c->spd.that.client
                           , &c->spd.this.host_addr, &c->spd.this.client
                           , 256
 			  , IPSEC_PROTO_ANY
                           , c->spd.this.protocol
-                          , SADB_SATYPE_UNSPEC
+                          , ET_UNSPEC
                           , null_proto_info, 0
                           , ERO_DEL_INBOUND, "delete inbound");
     }
@@ -1952,6 +1928,14 @@ init_kernel(void)
 	break;
 #endif
 
+#if defined(BSD_KAME) 
+    case USE_BSDKAME:
+	openswan_log("Using BSD/KAME IPsec interface code on %s"
+		     , kversion);
+	kernel_ops = &bsdkame_kernel_ops;
+	break;
+#endif
+
 #if defined(KLIPS_MAST) 
     case USE_MASTKLIPS:
 	openswan_log("Using KLIPSng (mast) IPsec interface code on %s"
@@ -2066,9 +2050,7 @@ install_inbound_ipsec_sa(struct state *st)
 	    return FALSE;
 	}
     }
-#if 0
     DBG_log("outgoing SA has refhim=%u", st->refhim);
-#endif
 
     /* (attempt to) actually set up the SAs */
     return setup_half_ipsec_sa(st, TRUE);
@@ -2314,7 +2296,7 @@ route_and_eroute(struct connection *c USED_BY_KLIPS
                     , bs->said.spi      /* network order */
                     , SA_INT            /* proto */
                     , 0                 /* transport_proto */
-                    , SADB_X_SATYPE_INT
+                    , ET_INT
                     , null_proto_info
                     , SHUNT_PATIENCE
                     , ERO_REPLACE, "restore");
