@@ -726,9 +726,7 @@ static void nat_traversal_ka_event_state (struct state *st, void *data)
 		st_newest = state_with_serialno(c->newest_isakmp_sa);
 		if ((st_newest)
 		    && ((st_newest->st_state==STATE_MAIN_R3)
-			|| (st_newest->st_state==STATE_MAIN_I4)
-			|| (st_newest->st_state == STATE_AGGR_R2)
-			|| (st_newest->st_state == STATE_AGGR_I2))
+			|| (st_newest->st_state==STATE_MAIN_I4))
 		    && (st_newest->hidden_variables.st_nat_traversal & NAT_T_DETECTED)
 		    && ((st_newest->hidden_variables.st_nat_traversal & LELEM(NAT_TRAVERSAL_NAT_BHND_ME))
 			|| (_force_ka)))
@@ -741,41 +739,6 @@ static void nat_traversal_ka_event_state (struct state *st, void *data)
 	    reset_cur_state();
 	    (*_kap_st)++;
 	}
-
-	if ( ((st->st_state == STATE_QUICK_R2)
-	      || (st->st_state == STATE_QUICK_I2))
-	     &&	(st->hidden_variables.st_nat_traversal & NAT_T_DETECTED)
-	     &&	((st->hidden_variables.st_nat_traversal & LELEM(NAT_TRAVERSAL_NAT_BHND_ME))
-		 || (_force_ka)))
-	{
-	    /**
-	     * - IPSEC SA established
-	     * - NAT-Traversal detected
-	     * - NAT-KeepAlive needed (we are NATed)
-	     */
-	    if (c->newest_ipsec_sa != st->st_serialno) {
-		/** 
-		 * if newest is also valid, ignore this one, we will only use
-		 * newest. 
-		 */
-		struct state *st_newest;
-		st_newest = state_with_serialno(c->newest_ipsec_sa);
-		if ((st_newest)
-		    && ((st_newest->st_state==STATE_QUICK_R2)
-			|| (st_newest->st_state == STATE_QUICK_I2))
-		    && (st_newest->hidden_variables.st_nat_traversal & NAT_T_DETECTED)
-		    && ((st_newest->hidden_variables.st_nat_traversal & LELEM(NAT_TRAVERSAL_NAT_BHND_ME))
-			|| (_force_ka)))
-		{
-		    return;
-		}
-	    }
-	    set_cur_state(st);
-	    nat_traversal_send_ka(st);
-	    reset_cur_state();
-	    (*_kap_st)++;
-	}
-	
 }
 
 void nat_traversal_ka_event (void)
@@ -795,58 +758,64 @@ void nat_traversal_ka_event (void)
 }
 
 struct _new_mapp_nfo {
-	struct state *st;
-	ip_address    addr;
-	u_int16_t     port;
+	ip_address addr;
+	u_int16_t sport, dport;
 };
 
-
 static void nat_traversal_find_new_mapp_state (struct state *st, void *data)
 {
+	struct connection *c = st->st_connection;
 	struct _new_mapp_nfo *nfo = (struct _new_mapp_nfo *)data;
 
-	if(st->st_serialno == nfo->st->st_clonedfrom ||
-	   st->st_clonedfrom == nfo->st->st_clonedfrom) {
-		char b1[ADDRTOT_BUF];
-		char b2[ADDRTOT_BUF];
-		struct connection *c = st->st_connection;
+	if ((c)
+	    && sameaddr(&st->st_remoteaddr, &(nfo->addr)) &&
+	    (st->st_remoteport == nfo->sport))
+	{
 
-		addrtot(&st->st_remoteaddr, 0, b1, ADDRTOT_BUF);
-		addrtot(&nfo->addr,         0, b2, ADDRTOT_BUF);
+	    /**
+	     * Change host port
+	     */
+	    st->st_remoteport = nfo->dport;
 
-		openswan_log("new NAT mapping for #%u, was %s:%d, now %s:%d"
-			     , (unsigned int)st->st_serialno
-			     , b1, st->st_remoteport
-			     , b2, nfo->port);
-		
-		/* update it */
-		st->st_remoteaddr = nfo->addr;
-		st->st_remoteport = nfo->port;
-		st->hidden_variables.st_natd = nfo->addr;
-
-		if(c->kind == CK_INSTANCE) {
-			c->spd.that.host_addr = nfo->addr;
+	    if (IS_IPSEC_SA_ESTABLISHED(st->st_state) ||
+		IS_ONLY_INBOUND_IPSEC_SA_ESTABLISHED(st->st_state)) {
+		if (!update_ipsec_sa(st)) {
+		    /**
+		     * If ipsec update failed, restore old port or we'll
+		     * not be able to update anymore.
+		     */
+		    st->st_remoteport = nfo->sport;
 		}
+	    }
 	}
 }
 
-
-static int nat_traversal_new_mapping(struct state *st
-				     , const ip_address *nsrc
-				     , u_int16_t nsrcport)
+static int nat_traversal_new_mapping(const ip_address *src, u_int16_t sport,
+	const ip_address *dst, u_int16_t dport)
 {
+	char srca[ADDRTOT_BUF], dsta[ADDRTOT_BUF];
 	struct _new_mapp_nfo nfo;
-	char ba[ADDRTOT_BUF];
-	
-	addrtot(nsrc, 0, ba, ADDRTOT_BUF);
 
-	DBG_log("state #%u NAT-T: new mapping %s:%d"
-		, (unsigned int)st->st_serialno
-		, ba, nsrcport);
+	addrtot(src, 0, srca, ADDRTOT_BUF);
+	addrtot(dst, 0, dsta, ADDRTOT_BUF);
 
-	nfo.st    = st;
-	nfo.addr  = *nsrc;
-	nfo.port  = nsrcport;
+	if (!sameaddr(src, dst)) {
+		loglog(RC_LOG_SERIOUS, "nat_traversal_new_mapping: "
+			"address change currently not supported [%s:%d,%s:%d]",
+			srca, sport, dsta, dport);
+		return -1;
+	}
+
+	if (sport == dport) {
+		/* no change */
+		return 0;
+	}
+
+	DBG_log("NAT-T: new mapping %s:%d/%d)", srca, sport, dport);
+
+	nfo.addr = *src;
+	nfo.sport = sport;
+	nfo.dport = dport;
 
 	for_each_state((void *)nat_traversal_find_new_mapp_state, &nfo);
 
@@ -863,17 +832,15 @@ void nat_traversal_change_port_lookup(struct msg_digest *md, struct state *st)
 
 	if (md) {
 		/**
-		 * If source port/address has changed, update (including other
+		 * If source port has changed, update (including other
 		 * states and established kernel SA)
 		 */
-		if (st->st_remoteport != md->sender_port ||
-		    !sameaddr(&st->st_remoteaddr, &md->sender)) {
-
-			nat_traversal_new_mapping(st
-						  , &md->sender
-						  , md->sender_port);
+		if (st->st_remoteport != md->sender_port) {
+		    nat_traversal_new_mapping(&st->st_remoteaddr
+					      , st->st_remoteport
+					      , &st->st_remoteaddr
+					      , md->sender_port);
 		}
-
 		/**
 		 * If interface type has changed, update local port (500/4500)
 		 */
@@ -941,6 +908,7 @@ void nat_traversal_change_port_lookup(struct msg_digest *md, struct state *st)
 	}
 }
 
+#ifdef KLIPS
 struct _new_klips_mapp_nfo {
 	struct sadb_sa *sa;
 	ip_address src, dst;
@@ -957,7 +925,9 @@ static void nat_t_new_klips_mapp (struct state *st, void *data)
 	    && sameaddr(&st->st_remoteaddr, &(nfo->src))
 	    && (st->st_esp.our_spi == nfo->sa->sadb_sa_spi))
 	{
-		nat_traversal_new_mapping(st, &(nfo->dst), nfo->dport);
+	    nat_traversal_new_mapping(&st->st_remoteaddr
+				      , st->st_remoteport
+				      , &(nfo->dst), nfo->dport);
 	}
 }
 
@@ -1014,6 +984,7 @@ void process_pfkey_nat_t_new_mapping(
 	if (ugh != NULL)
 		openswan_log("K_SADB_X_NAT_T_NEW_MAPPING message from KLIPS malformed: %s", ugh);
 }
+#endif /* USE_KLIPS */
 
 #endif
 
