@@ -46,12 +46,9 @@ typedef u_int32_t msgid_t;	/* Network order! */
 #define MAINMODE_MSGID    ((msgid_t) 0)
 
 struct state;	/* forward declaration of tag */
-extern void reserve_msgid(struct state *isakmp_sa, msgid_t msgid);
-extern bool unique_msgid(struct state *isakmp_sa, msgid_t msgid);
-
+extern bool reserve_msgid(struct state *isakmp_sa, msgid_t msgid);
 extern msgid_t generate_msgid(struct state *isakmp_sa);
 
-#define XAUTH_USERNAME_LEN 64
 
 /* Oakley (Phase 1 / Main Mode) transform and attributes
  * This is a flattened/decoded version of what is represented
@@ -135,6 +132,10 @@ struct hidden_variables {
     ip_address     st_natd;
 };                        
 
+#define set_suspended(st,md) do { st->st_suspended_md=md; \
+                                  st->st_suspended_md_func=__FUNCTION__; \
+                                  st->st_suspended_md_line=__LINE__; } while(0)
+
 /* state object: record the state of a (possibly nascent) SA
  *
  * Invariants (violated only during short transitions):
@@ -155,6 +156,8 @@ struct state
                                                 */
 
     struct msg_digest *st_suspended_md;        /* suspended state-transition */
+    const char        *st_suspended_md_func;
+    int                st_suspended_md_line;
 
     struct oakley_trans_attrs st_oakley;
 
@@ -183,7 +186,6 @@ struct state
     u_int16_t          st_localport;           
 
     msgid_t            st_msgid;               /* MSG-ID from header.  Network Order! */
-    bool               st_reserve_msgid;       /* if TRUE, then message id has been reserved already */
 
     msgid_t            st_msgid_phase15;       /* msgid for phase 1.5 */
     msgid_t            st_msgid_phase15b;      /* msgid for phase 1.5 */
@@ -277,8 +279,7 @@ struct state
 
     struct hidden_variables hidden_variables;
 
-    char                st_xauth_username[XAUTH_USERNAME_LEN];
-    chunk_t             st_xauth_password;
+    unsigned char *st_xauth_username;
 
     /* RFC 3706 Dead Peer Detection */
     time_t              st_last_dpd;            /* Time of last DPD transmit */
@@ -312,8 +313,6 @@ extern void state_eroute_usage(ip_subnet *ours, ip_subnet *his
 extern void delete_state(struct state *st);
 struct connection;	/* forward declaration of tag */
 extern void delete_states_by_connection(struct connection *c, bool relations);
-extern void delete_p2states_by_connection(struct connection *c);
-extern void rekey_p2states_by_connection(struct connection *c);
 
 extern struct state
     *duplicate_state(struct state *st),
