@@ -28,6 +28,7 @@ $failed=0;
 $passed=0;
 $missed=0;
 $total=0;
+$multicolumn=0;
 
 #$gnatsurl="http://gnats.freeswan.org/bugs/gnatsweb.pl?database=freeswan&amp;cmd=view+audit-trail&amp;pr=";
 $gnatsurl=undef;
@@ -39,15 +40,17 @@ sub htmlize_test {
 
   my($expected, $verdict, $packetstat, $consolestat, $file);
 
-  if(++$linecount > 40) {
-	print HTMLFILE "</TABLE>\n";
-	print HTMLFILE "<TD>";
-	print HTMLFILE "<TABLE>\n";
-
-	print HTMLFILE "<TR><TH COLSPAN=3>$testtypename tests</TH></TR>\n";
-	print HTMLFILE "<TR><TH>Test name</TH><TH>Result</TH><TH>Detail</TH></TR>\n";
- 	$linecount=4;
-  }	
+  if($multicolumn) {
+    if(++$linecount > 40) {
+      print HTMLFILE "</TABLE>\n";
+      print HTMLFILE "<TD>";
+      print HTMLFILE "<TABLE>\n";
+      
+      print HTMLFILE "<TR><TH COLSPAN=3>$testtypename tests</TH></TR>\n";
+      print HTMLFILE "<TR><TH>Test name</TH><TH>Result</TH><TH>Detail</TH></TR>\n";
+      $linecount=4;
+    }
+  }    
 
   print HTMLFILE "<TR><TD>";
 
@@ -340,7 +343,7 @@ print HTMLFILE "<TABLE border>\n";
 print HTMLFILE "<TD>";
 
 $testtypename="Regression";
-print HTMLFILE "<TABLE>\n";
+print HTMLFILE "<TABLE>\n" if ($multicolumn);
 print HTMLFILE "<TR><TH COLSPAN=3>Regression tests</TH></TR>\n";
 print HTMLFILE "<TR><TH>Test name</TH><TH>Result</TH><TH>Detail</TH></TR>\n";
 $linecount=3;
@@ -354,8 +357,8 @@ if($wanttestcategories) {
   }
   
   $testtypename="Goal";
-  print HTMLFILE "</TABLE>\n";
-  print HTMLFILE "<TABLE>\n";
+  print HTMLFILE "</TABLE>\n" if ($multicolumn);
+  print HTMLFILE "<TABLE>\n"  if ($multicolumn);
   print HTMLFILE "<TR><TH COLSPAN=3>Goal tests</TH></TR>\n";
   print HTMLFILE "<TR><TH>Test name</TH><TH>Result</TH><TH>Detail</TH></TR>\n";
   $linecount+=3;
@@ -368,8 +371,8 @@ if($wanttestcategories) {
   }
   
   $testtypename="Exploit ";
-  print HTMLFILE "</TABLE>\n";
-  print HTMLFILE "<TABLE>\n";
+  print HTMLFILE "</TABLE>\n"  if ($multicolumn);
+  print HTMLFILE "<TABLE>\n"   if ($multicolumn);
   print HTMLFILE "<TR><TH COLSPAN=3>Exploit tests</TH></TR>\n";
   print HTMLFILE "<TR><TH>Test name</TH><TH>Result</TH><TH>Detail</TH></TR>\n";
   $linecount+=3;
@@ -400,7 +403,7 @@ if($subtotal > 0) {
   $testrate="inf";
 }
 
-print HTMLFILE "</TABLE>  \n";
+print HTMLFILE "</TABLE>  \n"  if ($multicolumn);
 print HTMLFILE "</TABLE>  \n";
 print HTMLFILE "\n<BR><PRE>TOTAL tests: $total SKIPPED: $skipped   PASSED: $passed   FAILED: $failed   MISSED: $missed  SUCCESS RATE: $testrate%</PRE><BR>\n";
 print HTMLFILE "<A HREF=\"stdout.txt\">stdout</A><BR>\n";
